
class BaseFrequencyStrategy(object):


    def __init__(self, frequency_ratio_property, frequency_absolute_property, namespaces_dict):
        self._frequency_ratio_property = frequency_ratio_property
        self._frequency_absolute_property = frequency_absolute_property
        self._namespaces_dict = namespaces_dict

    def serialize_frequency(self, statement):
        raise NotImplementedError("This should be implemented in child classes")

    def annotations_for_frequency(self, statement):
        raise NotImplementedError("This should be implemented in child classes")




class FileLineReader(object):

    def __init__(self, source_file):
        self._source_file = source_file

    def read_lines(self):
        with open(self._source_file, "r", errors='ignore') as in_stream:
            for a_line in in_stream:
                yield a_line


import unittest
from shexer.shaper import Shaper
from test.const import BASE_FILES, default_namespaces
from test.t_utils import file_vs_str_tunned_comparison
import os.path as pth
from shexer.consts import TURTLE

_BASE_DIR = BASE_FILES + "disable_or" + pth.sep

class TestDisableOrStatements(unittest.TestCase):

    def test_or_enabled_choice_useful_IRI(self):
        shaper = Shaper(graph_file_input=_BASE_DIR + "g3_or_example.ttl",
                        namespaces_dict=default_namespaces(),
                        all_classes_mode=True,
                        input_format=TURTLE,
                        disable_comments=True,
                        disable_or_statements=False)
        str_result = shaper.shex_graph(string_output=True)
        # In case the choice includes the IRI macro, then no OR should appear
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "or_disabled.shex",
                                                      str_target=str_result,
                                                      or_shapes=True))

    def test_or_enabled_choice_expendable_IRI(self):
        shaper = Shaper(graph_file_input=_BASE_DIR + "g_or_example_expandable_IRI.ttl",
                        namespaces_dict=default_namespaces(),
                        all_classes_mode=True,
                        input_format=TURTLE,
                        disable_comments=True,
                        disable_or_statements=False)
        str_result = shaper.shex_graph(string_output=True)
        # In case the choice includes the IRI macro, then no OR should appear
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "or_enabled.shex",
                                                      str_target=str_result,
                                                      or_shapes=True))

    def test_or_disabled(self):
        shaper = Shaper(graph_file_input=_BASE_DIR + "g3_or_example.ttl",
                        namespaces_dict=default_namespaces(),
                        all_classes_mode=True,
                        input_format=TURTLE,
                        disable_comments=True,
                        disable_or_statements=True)
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "or_disabled.shex",
                                                      str_target=str_result,
                                                      or_shapes=True))


from shexer.utils.target_elements import determine_original_target_nodes_if_needed
from shexer.model.property import Property
from shexer.utils.uri import remove_corners
from shexer.consts import SHAPES_DEFAULT_NAMESPACE, SHAPE_EXAMPLES, ALL_EXAMPLES
from shexer.core.profiling.consts import POS_CLASSES
from shexer.utils.log import log_msg
from shexer.utils.uri import longest_common_prefix
from shexer.core.profiling.strategy.direct_features_strategy import DirectFeaturesStrategy
from shexer.core.profiling.strategy.include_reverse_features_strategy import IncludeReverseFeaturesStrategy
from shexer.core.profiling.consts import RDF_TYPE_STR
from shexer.utils.structures.dicts import ShapeExampleFeaturesDict
from shexer.model.shape import STARTING_CHAR_FOR_SHAPE_NAME

_MINIMAL_IRI_INIT = STARTING_CHAR_FOR_SHAPE_NAME




class ClassProfiler(object):

    def __init__(self, triples_yielder, instances_dict, instantiation_property_str=RDF_TYPE_STR,
                 remove_empty_shapes=True, original_target_classes=None, original_shape_map=None,
                 shapes_namespace=SHAPES_DEFAULT_NAMESPACE, inverse_paths=False, detect_minimal_iri=False,
                 examples_mode=None, namespaces_dict=None):
        self._triples_yielder = triples_yielder
        self._instances_dict = instances_dict  # TODO  refactor: change name once working again
        # self._instances_shape_dict = {}
        self._shapes_namespace = shapes_namespace
        self._shape_names_dict = {}  # Will be filled during execution
        self._relevant_triples = 0
        self._instantiation_property_str = self._decide_instantiation_property(instantiation_property_str)
        self._remove_empty_shapes = remove_empty_shapes
        self._original_raw_target_classes = original_target_classes
        self._classes_shape_dict = {}  # Will be filled later
        self._class_counts = {}  # Will be filled later
        self._detect_minimal_iri = detect_minimal_iri
        self._examples_mode = examples_mode
        self._inverse_paths = inverse_paths
        self._namespaces_dict = namespaces_dict if namespaces_dict is not None else {}
        self._original_shape_map = original_shape_map

        self._original_target_nodes = None # Will be filled during execution

        if detect_minimal_iri or examples_mode is not None:
            self._shape_feature_examples = ShapeExampleFeaturesDict(track_inverse_features=inverse_paths)
            # This last one will be filled later if detect_minimal_iri is True
        self._strategy = DirectFeaturesStrategy(class_profiler=self) if not inverse_paths \
            else IncludeReverseFeaturesStrategy(class_profiler=self)



    def profile_classes(self, verbose):
        log_msg(verbose=verbose,
                msg="Starting class profiler...")
        self._init_class_counts_and_shape_dict()
        log_msg(verbose=verbose,
                msg="Instance counts completed. Annotating instance features...")
        self._adapt_instances_dict()
        self._build_shape_of_instances()
        log_msg(verbose=verbose,
                msg="Instance features annotated. Number of relevant triples computed: {}. "
                    "Building shape profiles...".format(self._relevant_triples))

        self._build_class_profile()
        log_msg(verbose=verbose,
                msg="Draft shape profiles built. Cleaning shape profiles...")
        self._clean_class_profile()
        log_msg(verbose=verbose,
                msg="Shape profiles done. Working with {} shapes.".format(len(self._classes_shape_dict)))
        if self._detect_minimal_iri or self._examples_mode in [SHAPE_EXAMPLES, ALL_EXAMPLES]:
            log_msg(verbose=verbose,
                    msg="Detecting example features for each shape...")
            self._init_anotation_example_method()
            self._detect_example_features()
            log_msg(verbose=verbose,
                    msg="Mimimal IRIs detected...")
        return self._classes_shape_dict, self._class_counts, \
            self._shape_feature_examples if (self._detect_minimal_iri or self._examples_mode is not None) else None,\
            self._shape_names_dict

    def get_target_classes_dict(self):
        return self._instances_dict

    def _detect_example_features(self):
        self._init_class_features_dict()
        self._annotate_example_features()


    def _init_class_features_dict(self):
        for a_class_key in self._class_counts:
            self._shape_feature_examples.set_shape_min_iri(shape_id=a_class_key,
                                                           min_iri=_MINIMAL_IRI_INIT)

    def _init_anotation_example_method(self):
        if self._detect_minimal_iri and self._examples_mode in [SHAPE_EXAMPLES, ALL_EXAMPLES]:
            self._annotate_example_features = self._annotate_shape_examples_and_min_iris
        elif self._detect_minimal_iri:
            self._annotate_example_features = self._annotate_min_iris
        else:  # not minimal IRIs, but if this was called, at this point, it means that we are looking for shape examples
            self._annotate_example_features = self._annotate_shape_examples
    def _annotate_example_features(self):
        raise NotImplementedError()

    def _annotate_min_iris(self):
        for an_instance_iri in self._instances_dict:
            for a_class_key in self._instances_dict[an_instance_iri][POS_CLASSES]:
                self._update_shape_min_iri(target_shape=a_class_key,
                                           instance_iri=an_instance_iri)
    def _annotate_shape_examples(self):
        for an_instance_iri in self._instances_dict:
            for a_class_key in self._instances_dict[an_instance_iri][POS_CLASSES]:
                if self._shape_feature_examples.shape_example(shape_id=a_class_key) is None:
                    self._shape_feature_examples.set_shape_example(shape_id=a_class_key,
                                                           example_iri=an_instance_iri)

    def _annotate_shape_examples_and_min_iris(self):
        for an_instance_iri in self._instances_dict:
            for a_class_key in self._instances_dict[an_instance_iri][POS_CLASSES]:
                self._update_shape_min_iri(target_shape=a_class_key,
                                           instance_iri=an_instance_iri)
                if self._shape_feature_examples.shape_example(shape_id=a_class_key) is None:
                    self._shape_feature_examples.set_shape_example(shape_id=a_class_key,
                                                           example_iri=an_instance_iri)


    def _update_shape_min_iri(self, target_shape, instance_iri):
        curr_iri = self._shape_feature_examples.shape_min_iri(shape_id=target_shape)
        if curr_iri == _MINIMAL_IRI_INIT:
            self._shape_feature_examples.set_shape_min_iri(shape_id=target_shape,
                                                           min_iri=instance_iri)
            return

        self._shape_feature_examples.set_shape_min_iri(shape_id=target_shape,
                                                       min_iri=longest_common_prefix(uri1=instance_iri,
                                                                                     uri2=curr_iri))

    @staticmethod
    def _decide_instantiation_property(instantiation_property_str):
        if instantiation_property_str == None:
            return RDF_TYPE_STR
        if type(instantiation_property_str) == Property:
            return str(instantiation_property_str)
        if type(instantiation_property_str) == str:
            return remove_corners(a_uri=instantiation_property_str,
                                  raise_error_if_no_corners=False)
        raise ValueError("Unrecognized param type to define instantiation property")


    def _init_class_counts_and_shape_dict(self):
        """
        IMPORTANT: this method should be called before adapting the instances_dict

        :return:
        """
        self._init_original_targets()
        self._init_annotated_targets()


    def _init_annotated_targets(self):
        self._strategy.init_annotated_targets()

    def _init_original_targets(self):
        self._strategy.init_original_targets()

    def _build_class_profile(self):
        for an_instance in self._instances_dict:
            self._strategy.annotate_instance_features(an_instance)

    def _clean_class_profile(self):
        if not self._remove_empty_shapes:
            return
        self._original_target_nodes = determine_original_target_nodes_if_needed(remove_empty_shapes=self._remove_empty_shapes,
                                                                                original_target_classes=self._original_raw_target_classes,
                                                                                original_shape_map=self._original_shape_map,
                                                                                shape_names_dict=self._shape_names_dict)
        shapes_to_remove = self._detect_shapes_to_remove()

        while len(shapes_to_remove) != 0:
            self._iteration_remove_empty_shapes(shapes_to_remove)
            shapes_to_remove = self._detect_shapes_to_remove()

    def _detect_shapes_to_remove(self):
        shapes_to_remove = set()
        for a_shape_key in self._classes_shape_dict:
            if not self._is_original_target_shape(a_shape_key):
                if not self._has_it_annotated_features(a_shape_key):
                    shapes_to_remove.add(a_shape_key)
        return shapes_to_remove

    def _is_original_target_shape(self, shape_label):
        return shape_label in self._original_target_nodes

    def _has_it_annotated_features(self, shape_label):
        return self._strategy.has_shape_annotated_features(shape_label)

    def _iteration_remove_empty_shapes(self, target_shapes):
        for a_shape_label_key in self._classes_shape_dict:
            for a_prop_key in self._classes_shape_dict[a_shape_label_key]:
                # print(self._classes_shape_dict[a_shape_label_key][a_prop_key])
                for a_shape_to_remove in target_shapes:
                    if a_shape_to_remove in self._classes_shape_dict[a_shape_label_key][a_prop_key]:
                        del self._classes_shape_dict[a_shape_label_key][a_prop_key][a_shape_to_remove]
        for a_shape_to_remove in target_shapes:
            if a_shape_to_remove in self._classes_shape_dict:
                del self._classes_shape_dict[a_shape_to_remove]

    def _build_shape_of_instances(self):
        for a_triple in self._yield_relevant_triples():
            self._relevant_triples += 1
            self._annotate_feature_of_target_instance(a_triple)

    def _annotate_feature_of_target_instance(self, a_triple):
        self._strategy.annotate_triple_features(a_triple)

    def _adapt_instances_dict(self):
        self._strategy.adapt_instances_dict()

    def _adapt_entry_dict_if_needed(self, str_subj):
        if type(self._instances_dict[str_subj]) == list:
            self._instances_dict[str_subj] = (self._instances_dict[str_subj], {})

    def _yield_relevant_triples(self):
        for a_triple in self._triples_yielder.yield_triples():
            if self._strategy.is_a_relevant_triple(a_triple):
                yield a_triple

    # def _set_anotation_instance_methods(self):
    #     # MIN IRIS
    #     if self._detect_minimal_iri:
    #         self._update_shape_min_iri = self._update_shape_min_iri_active
    #     else:
    #         self._update_shape_min_iri = self._update_shape_min_iri_inactive
    #
    #     # EXAMPLE FEATURES
    #     if self._examples_mode is None:
    #         self._update_shape_examples = self._update_shape_examples_inactive
    #     elif self._examples_mode == SHAPE_EXAMPLES:
    #         self._update_shape_examples = self._update_shape_examples_only_shapes
    #     elif self._examples_mode == CONSTRAINT_EXAMPLES:
    #         self._update_shape_examples = self._update_shape_examples_only_constraints
    #     elif self._examples_mode == ALL_EXAMPLES:
    #         self._update_shape_examples = self._update_shape_examples_shapes_and_constraints
    #     else:
    #         raise ValueError("Unrecognized mode for getting shape examples. Choose one between the values offered in shexer.const, section # EXAMPLES")


    #
    # def _update_shape_examples_only_constraints(self, instance_id, shape_id):
    #     self._strategy.look_for_example_features(instance_id=instance_id,
    #                                              shape_id=shape_id)
    #
    # def _update_shape_examples_shapes_and_constraints(self, instance_id, shape_id):
    #     self._update_shape_examples_only_shapes(instance_id, shape_id)
    #     self._update_shape_examples_only_constraints(instance_id, shape_id)
    #
    # def _update_shape_examples(self, instance_id, shape_id):
    #     raise NotImplementedError()
    #
    # def _update_shape_examples_inactive(self, instance_id, shape_id):
    #     pass  # This is OK, do nothing
    #







from SPARQLWrapper import SPARQLWrapper, JSON
from urllib.error import HTTPError
from time import sleep
import ssl
ssl._create_default_https_context = ssl._create_unverified_context

from SPARQLWrapper.SPARQLExceptions import EndPointInternalError

_FAKE_USER_AGENT = "Mozilla/5.0 (Platform; Security; OS-or-CPU; Localization; rv:1.4) Gecko/20030624 Netscape/7.1 (ax)"
_RESULTS_KEY = "results"
_BINDINGS_KEY = "bindings"
_VALUE_KEY = "value"
_TYPE_KEY = "type"
_URI_TYPE = "uri"

_XML_LANG_FIELD = "xml:lang"


def _add_lang_if_needed(result_dict):
    result = result_dict[_VALUE_KEY]
    if _XML_LANG_FIELD in result_dict:
        result += '"' + result + '"@' + result_dict[_XML_LANG_FIELD]
    return result


def _add_corners_if_needed(target_elem, elem_type):
    if elem_type == _URI_TYPE and not target_elem.startswith("<"):
        return "<" + target_elem + ">"
    return target_elem


def query_endpoint_single_variable(endpoint_url, str_query, variable_id, max_retries=10, sleep_time=5, fake_user_agent=True):
    """
    It receives an SPARQL query with a single variable and returns a list with the resulting nodes

    :param endpoint_url:
    :param str_query:
    :param variable_id:
    :return:
    """
    result_query = _query_endpoint_json_result(endpoint_url=endpoint_url,
                                               str_query=str_query,
                                               max_retries=max_retries,
                                               sleep_time=sleep_time,
                                               fake_user_agent=fake_user_agent)
    result = []
    for row in result_query[_RESULTS_KEY][_BINDINGS_KEY]:
        an_elem = row[variable_id][_VALUE_KEY]
        result.append(an_elem)
    return result


def query_endpoint_sp_of_an_o(endpoint_url, str_query, s_id, p_id, max_retries=5, sleep_time=2, fake_user_agent=True):
    result_query = _query_endpoint_json_result(endpoint_url=endpoint_url,
                                               str_query=str_query,
                                               max_retries=max_retries,
                                               sleep_time=sleep_time,
                                               fake_user_agent=fake_user_agent)
    result = []
    for row in result_query[_RESULTS_KEY][_BINDINGS_KEY]:
        p_value = _add_corners_if_needed(target_elem=row[p_id][_VALUE_KEY],
                                         elem_type=row[p_id][_TYPE_KEY])
        s_value = _add_corners_if_needed(target_elem=_add_lang_if_needed(row[s_id]),
                                         elem_type=row[s_id][_TYPE_KEY])
        result.append((s_value, p_value))
    return result


def query_endpoint_po_of_an_s(endpoint_url, str_query, p_id, o_id, max_retries=5, sleep_time=2, fake_user_agent=True):

    result_query = _query_endpoint_json_result(endpoint_url=endpoint_url,
                                               str_query=str_query,
                                               max_retries=max_retries,
                                               sleep_time=sleep_time,
                                               fake_user_agent=fake_user_agent)
    result = []
    for row in result_query[_RESULTS_KEY][_BINDINGS_KEY]:
        if p_id in row and o_id in row:
            p_value = _add_corners_if_needed(target_elem=row[p_id][_VALUE_KEY],
                                             elem_type=row[p_id][_TYPE_KEY])
            o_value = _add_corners_if_needed(target_elem=_add_lang_if_needed(row[o_id]),
                                             elem_type=row[o_id][_TYPE_KEY])
            result.append((p_value, o_value))
    return result



def _query_endpoint_json_result(endpoint_url, str_query, max_retries=5, sleep_time=2, fake_user_agent=True):
    first_failure = True
    sparql = SPARQLWrapper(endpoint_url)
    if fake_user_agent:
        sparql.agent = _FAKE_USER_AGENT
    sparql.setQuery(str_query)
    sparql.setReturnFormat(JSON)
    last_error = None
    while max_retries > 0:
        try:
            return sparql.query().convert()
        except (HTTPError, EndPointInternalError) as e:
            max_retries -= 1
            sleep(sleep_time)
            last_error = e
            if first_failure and not fake_user_agent:
                sparql.agent = _FAKE_USER_AGENT
                first_failure = not first_failure
    last_error.msg = "Max number of attempt reached, it is not possible to perform the query. Msg:\n" + last_error.msg


class FederatedSource(object):
    def __init__(self, endpoint_url, property_link, link_in_federated_source, alias, origin_position_in_triple=0):
        """
        :param endpoint_url: (str) URL where the content is exposed online to be queried.
        :param property_link: (str) URI of the property used to link elements in different sources (owl:sameAs, schema:sameAs, rdfs:seeAlso...)
        :param link_in_federated_source: (bool) it indicates if the link between two entities is expected to be found in the origin source (False)
                                         or in the federated source itself (True)
        :param alias: (str) this string will be used to give proper names to extracted shapes.
        :param origin_position_in_triple: (int) it indicates where is the original node expected to be found in the linked triples. 0 means subject, 1 means object
        """
        self._endpoint_url = endpoint_url
        self._property_link = property_link
        self._link_in_federated_source = link_in_federated_source
        self._alias = alias
        self._origin_position_in_triple = origin_position_in_triple
        self._set_of_instances = None  # It will be filled whn tracking instances


    @property
    def endpoint_url(self):
        return self._endpoint_url

    @property
    def origin_position_in_triple(self):
        return self._origin_position_in_triple

    @property
    def property_link(self):
        return self._property_link

    @property
    def link_in_federated_source(self):
        return self._link_in_federated_source

    @property
    def alias(self):
        return self._alias

    @property
    def set_of_instances(self):
        return self._instances_dict

    @set_of_instances.setter
    def set_of_instances(self, a_dict):
        self._instances_dict = a_dict




from shexer.core.shexing.strategy.minimal_iri_strategy.abstract_min_iri_strategy import AbstractMinIriStrategy
import re

_SEP_CHARS = re.compile("[:/#]")


class AnnotateMinIriStrategy(AbstractMinIriStrategy):

    def __init__(self, min_iris_dict):
        self._min_iris_dict = min_iris_dict

    def annotate_shape_iri(self, shape):
        self._min_iris_dict.set_shape_min_iri(shape_id=shape.class_uri,
                                              min_iri=self._determine_suitable_iri_pattern
                                                  (
                                                  self._min_iris_dict.shape_min_iri
                                                      (
                                                      shape.class_uri
                                                      )
                                                  )
                                              )
        # shape.iri_pattern = self._determine_suitable_iri_pattern(self._min_iris_dict.shape_min_iri(shape.class_uri))
        # shape.iri_pattern = self._determine_suitable_iri_pattern(self._min_iris_dict[shape.class_uri])

    def _determine_suitable_iri_pattern(self, longest_common_prefix):
        backwards_str = longest_common_prefix[::-1]
        last_sep_char = _SEP_CHARS.search(backwards_str)
        if last_sep_char is None:
            return None
        candidate_min_iri = backwards_str[last_sep_char.start():][::-1]
        if len(candidate_min_iri) < 3:  # Just too short. Kind of an arbitrary number
            return None
        if candidate_min_iri.startswith("http") and len(candidate_min_iri) < 8:  # http:// or https:// + an extra char
            return None
        return candidate_min_iri  # Let's say it is a worthy one



from shexer.core.instances.annotators.strategy_mode.base_strategy_mode import BaseStrategyMode
from shexer.core.instances.pconsts import _S, _P, _O
from shexer.core.instances.annotators.strategy_mode.instances_cap_exception import InstancesCapException

class InstanceCapMode(BaseStrategyMode):

    def __init__(self, annotator_ref, internal_strategy, instance_limit, n_target_classes = -1):
        super().__init__(annotator_ref)
        self._internal_strategy = internal_strategy
        self._instance_limit = instance_limit
        self._class_counts = {}
        self._n_target_classes = n_target_classes
        self._n_classes_completed = 0

        self.annotate_class = self._annotate_class_with_stop_condition if n_target_classes > 0 \
            else self._annotate_class_with_no_stop_condition


    def is_relevant_triple(self, a_triple):
        return self._check_class_counts(a_triple) and self._internal_strategy.is_relevant_triple(a_triple)

    def _check_class_counts(self, a_triple):
        """
        It returns False when it receives an instantiation triple whose class has already reached the maz number of
        instances allowed.

        :param a_triple:
        :return:
        """
        if a_triple[_P] != self._instantiation_property:
            return True
        if a_triple[_O].iri not in self._class_counts:
            return True
        if self._class_counts[a_triple[_O].iri] < self._instance_limit:
            return True
        return False


    def annotate_triple(self, a_triple):
        if self._instance_tracker.is_an_instantiation_prop(a_triple[_P]):
            self._annotator_ref.add_instance_to_instances_dict(a_triple)
            self.annotate_class(a_triple)

    def annotate_class(self, a_triple):
        raise NotImplementedError()


    def _annotate_class_with_stop_condition(self, a_triple):
        self._instances_dict[a_triple[_S].iri].append(a_triple[_O].iri)
        if a_triple[_O].iri not in self._class_counts:
            self._class_counts[a_triple[_O].iri] = 0
        self._class_counts[a_triple[_O].iri] += 1
        if self._class_counts[a_triple[_O].iri] == self._instance_limit:
            self._n_classes_completed += 1
        if self._n_classes_completed == self._n_target_classes:
            raise InstancesCapException()

    def _annotate_class_with_no_stop_condition(self, a_triple):
        self._instances_dict[a_triple[_S].iri].append(a_triple[_O].iri)
        if a_triple[_O].iri not in self._class_counts:
            self._class_counts[a_triple[_O].iri] = 0
        self._class_counts[a_triple[_O].iri] += 1


import traceback
from flask import Flask, request
from flask_cors import CORS
from shexer.consts import NT, RDF_TYPE
from shexer.shaper import Shaper
from shexer.utils.uri import remove_corners
import json
import sys

################ CONFIG

# PORT = 8080
HOST = "0.0.0.0"
MAX_LEN = 100000


################ Default namespace

default_namespaces = {"http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
                      "http://www.w3.org/2000/01/rdf-schema#": "rdfs",
                      "http://www.w3.org/2001/XMLSchema#": "xsd",
                      "http://www.w3.org/XML/1998/namespace/": "xml",
                      "http://www.w3.org/2002/07/owl#": "owl"
                      }


################ PARAM NAMES

TARGET_CLASSES_PARAM = "target_classes"
"""
List of strings: List of target classes to associate a shape with
"""

TARGET_GRAPH_PARAM = "raw_graph"
"""
String: RDF content to be analyzed
"""

INPUT_FORMAT_PARAM = "input_format"
"""
String: RDF syntax used. Ntriples is used by default Accepted values -->

"nt" (n-triples)
"turtle" (turtle)
"xml" (RDF/XML)
"n3" (n3)
"json-ld" (JSON LD)
"tsv_spo" (lines with subject predicate and object separated by tab '\\t' chars 
"""


INSTANTIATION_PROPERTY_PARAM = "instantiation_prop"
"""
String: property used to links an instance with its class. rdf:type by default.
"""


NAMESPACES_TO_IGNORE_PARAM = "ignore"
"""
List of Strings: List of namespaces whose properties should be ignored during the shexing process.
"""

INFER_NUMERIC_TYPES_PARAM = "infer_untyped_nums"
"""
Bool: default, True. If True, it tries to infer the numeric type (xsd:int, xsd:float..) of 
untyped numeric literals 
"""

DISCARD_USELESS_CONSTRAINTS_PARAM = "discard_useless_constraints"
"""
Bool: default, True. default, True. If True, it keeps just the most possible specific constraint w.r.t. cardinality 
"""

ALL_INSTANCES_COMPLIANT_PARAM = "all_compliant"
"""
Bool: default, True. default, True. If False, the shapes produced may not be compliant with all the entities considered
to build them. This is because it won't use Kleene closeres for any constraint. 
"""

KEEP_LESS_SPECIFIC_PARAM = "keep_less_specific"
"""
Bool: default, True. It prefers to use "+" closures rather than exact cardinalities in the triple constraints
"""

ACEPTANCE_THRESHOLD_PARAM = "threshold"
"""
Float: number in [0,1] that indicates the minimum proportion of entities that should have a given feature for this
to be accepted as a triple constraint in the produced shape.
"""

ALL_CLASSES_MODE_PARAM = "all_classes"
"""
Bool: default, False. If True, it generates a shape for every elements with at least an instance 
in the considered graph.
"""
SHAPE_MAP_PARAM = "shape_map"
"""
String: shape map to associate nodes with shapes. It uses the same syntax of validation shape maps. 
"""

REMOTE_GRAPH_PARAM = "graph_url"
"""
String: URL to retrieve an online raw graph.
"""

ENDPOINT_GRAPH_PARAM = "endpoint"
"""
String: URL of an SPARQL endpoint.
"""

NAMESPACES_PARAM = "prefixes"
"""
Dict. key are namespaces and values are prefixes. The pairs key value provided here will be used 
to parse the RDF content and t write the resulting shapes.
"""

QUERY_DEPTH_PARAM = "query_depth"
"""
Integer: default, 1. It indicates the depth to generate queries when targeting a SPARQL endpoint.
Currently it can be 1 or 2.
"""

DISABLE_COMMENTS_PARAM = "disable_comments"
"""
Bool: default, False. When set to True, the shapes do not include comment 
with ratio of entities compliant with a triple constraint
"""

QUALIFIER_NAMESPACES_PARAM = "namespaces_for_qualifiers"
"""
List. Default, None. When a list with elements is provided, the properties in the namespaces specified are considered
to be pointers to qualifier nodes.
"""

SHAPE_QUALIFIERS_MODE_PARAM = "shape_qualifiers_mode"
"""
Bool: default, False. When set to true, a shape is generated for those nodes detected as qualifiers according to
Wikidata data model and the properties pointing to them specified in QUALIFIER_NAMESPACES_PARAM
"""




################ SUPPORT FUNCTIONS


def _build_namespaces_dict(new_prefixes, defaults):
    """
    It merges the default list of namespaces with a

    :param new_prefixes:
    :param defaults:
    :return:
    """
    for a_key in new_prefixes:
        defaults[a_key] = new_prefixes[a_key]
    return defaults


def _jsonize_response(response):
    result = json.dumps({'result' : response})
    return result
    # result = {'result' : response}
    # return json.dumps(result)
    # return response


def _return_json_error_pool(error_pool):
    result = '{"Errors" : ['
    result += '"' + error_pool[0] + '"'
    for i in range(1, len(error_pool)):
        result += ', "' + error_pool[i] + '"'
    result += "]}"
    return _jsonize_response(result)


def _missing_param_error(param):
    return "Missing mandatory param: " + param


def _parse_endpoint_sparql(data, error_pool):
    if ENDPOINT_GRAPH_PARAM not in data:
        return None
    if type(data[ENDPOINT_GRAPH_PARAM]) != str:
        error_pool.append("You must provide a URL (string) in the field " + ENDPOINT_GRAPH_PARAM)
        return None
    return str(data[ENDPOINT_GRAPH_PARAM])


def _parse_remote_graph(data, error_pool):
    if REMOTE_GRAPH_PARAM not in data:
        return None
    if type(data[REMOTE_GRAPH_PARAM]) != str:
        error_pool.append("You must provide a URL (string) in the field " + REMOTE_GRAPH_PARAM)
        return None
    return str(data[REMOTE_GRAPH_PARAM])


def _parse_shape_map(data, error_pool):
    if SHAPE_MAP_PARAM not in data:
        return None
    if type(data[SHAPE_MAP_PARAM]) != str:
        error_pool.append("You must provide a string containing the shape map")
        return
    return str(data[SHAPE_MAP_PARAM])


def _parse_namespaces_to_ignore(data, error_pool):
    if NAMESPACES_TO_IGNORE_PARAM not in data:
        return None
    if type(data[NAMESPACES_TO_IGNORE_PARAM]) != list:
        error_pool.append("You must provide a non-empty list of URIs (string) in " + NAMESPACES_TO_IGNORE_PARAM)
        return None
    if len(data[NAMESPACES_TO_IGNORE_PARAM]) == 0 or type(data[NAMESPACES_TO_IGNORE_PARAM][0]) != str:
        error_pool.append("You must provide a non-empty list of URIs (string) in " + NAMESPACES_TO_IGNORE_PARAM)
        return
    return [str(a_uri) for a_uri in data[NAMESPACES_TO_IGNORE_PARAM]]


def _parse_namespaces_for_qualifiers(data, error_pool):
    if QUALIFIER_NAMESPACES_PARAM not in data:
        return None
    if type(data[QUALIFIER_NAMESPACES_PARAM]) != list:
        error_pool.append("You must provide a non-empty list of URIs (string) in " + QUALIFIER_NAMESPACES_PARAM)
        return None
    if len(data[QUALIFIER_NAMESPACES_PARAM]) == 0 or type(data[QUALIFIER_NAMESPACES_PARAM][0]) != str:
        error_pool.append("You must provide a non-empty list of URIs (string) in " + QUALIFIER_NAMESPACES_PARAM)
        return
    return [str(a_uri) for a_uri in data[QUALIFIER_NAMESPACES_PARAM]]



def _parse_namespaces(data, error_pool):
    if NAMESPACES_PARAM not in data:
        return {}
    if type(data[NAMESPACES_PARAM]) != dict:
        error_pool.append("You must provide a dict namespace_URI --> prefix  in " + NAMESPACES_PARAM)
        return {}
    if len(data[NAMESPACES_PARAM]) == 0:
        error_pool.append("You must provide a dict namespace_URI --> prefix  in " + NAMESPACES_PARAM)
        return {}
    result = {}
    for a_key in data[NAMESPACES_PARAM]:
        result[str(a_key)] = str(data[NAMESPACES_PARAM][a_key])
    return result


def _parse_target_classes(data, error_pool):
    if TARGET_CLASSES_PARAM not in data:
        return None
    if type(data[TARGET_CLASSES_PARAM]) != list:
        error_pool.append("You must provide a non-empty list of URIs (string) in " + TARGET_CLASSES_PARAM)
        return
    if len(data[TARGET_CLASSES_PARAM]) == 0 or type(data[TARGET_CLASSES_PARAM][0]) != str:
        error_pool.append("You must provide a non-empty list of URIs (string) in " + TARGET_CLASSES_PARAM)
        return
    return [str(a_uri) for a_uri in data[TARGET_CLASSES_PARAM]]


def _parse_graph(data, error_pool):
    if TARGET_GRAPH_PARAM not in data:
        return None
    if type(data[TARGET_GRAPH_PARAM]) != str:
        error_pool.append("You must provide a str containing an RDF graph ")
        return
    if len(data[TARGET_GRAPH_PARAM]) > MAX_LEN:
        error_pool.append("The size of the graphic is too big for this deployment. Introduce a graph using less than "
                          + str(MAX_LEN) + " chars")
        return
    return str(data[TARGET_GRAPH_PARAM])


def _parse_str_param(data, error_pool, key, default_value, opt_message=""):
    result = default_value
    if key in data:
        if type(data[key]) == str:
            result = data[key]
        else:
            error_pool.append(key + " must be a str. " + opt_message)
            return
    return result


def _parse_bool_param(data, error_pool, key, default_value, opt_message=""):
    result = default_value
    if key in data:
        if type(data[key]) == bool:
            result = data[key]
        else:
            error_pool.append(key + " must be 'true' or 'false'. " + opt_message)
            return

    return result


def _parse_input_format(data, error_pool):
    return _parse_str_param(data=data, error_pool=error_pool, key=INPUT_FORMAT_PARAM, default_value=NT)


def _parse_instantiation_prop(data, error_pool):
    candidate = _parse_str_param(data=data, error_pool=error_pool, key=INSTANTIATION_PROPERTY_PARAM,
                                 default_value=RDF_TYPE,
                                 opt_message="The default value is rdf:type")
    return remove_corners(a_uri=candidate,
                          raise_error_if_no_corners=False)


def _parse_infer_untyped_num(data, error_pool):
    return _parse_bool_param(data=data, error_pool=error_pool, key=INFER_NUMERIC_TYPES_PARAM, default_value=True,
                             opt_message="The default value is True")


def _parse_discard_useless(data, error_pool):
    return _parse_bool_param(data=data, error_pool=error_pool, key=DISCARD_USELESS_CONSTRAINTS_PARAM, default_value=True,
                             opt_message="The default value is True")


def _parse_all_compliant(data, error_pool):
    return _parse_bool_param(data=data, error_pool=error_pool, key=ALL_INSTANCES_COMPLIANT_PARAM,
                             default_value=True,
                             opt_message="The default value is True")


def _parse_all_classes_mode(data, error_pool):
    return _parse_bool_param(data=data, error_pool=error_pool, key=ALL_CLASSES_MODE_PARAM,
                             default_value=False,
                             opt_message="The default value is False")

def _parse_shape_qualifiers_mode(data, error_pool):
    return _parse_bool_param(data=data, error_pool=error_pool, key=SHAPE_QUALIFIERS_MODE_PARAM,
                             default_value=False,
                             opt_message="The default value is False")

def _parse_disable_comments(data, error_pool):
    return _parse_bool_param(data=data, error_pool=error_pool, key=DISABLE_COMMENTS_PARAM,
                             default_value=False,
                             opt_message="The default value is False")


def _parse_keep_less_specific(data, error_pool):
    return _parse_bool_param(data=data, error_pool=error_pool, key=ALL_INSTANCES_COMPLIANT_PARAM,
                             default_value=True,
                             opt_message="The default value is True")


def _parse_threshold(data, error_pool):
    if ACEPTANCE_THRESHOLD_PARAM in data:
        try:
            result = float(data[ACEPTANCE_THRESHOLD_PARAM])
            if result < 0 or result > 1:
                raise ValueError()
            return result
        except BaseException as e:
            error_pool.append(ACEPTANCE_THRESHOLD_PARAM + " must contain a float number in [0,1]. The default value is 0.")
    return 0.0


def _parse_query_depth(data, error_pool):
    if QUERY_DEPTH_PARAM in data:
        try:
            result = int(data[QUERY_DEPTH_PARAM])
            if result > 2 or result < 1:
                raise ValueError()
            return result
        except BaseException as e:
            error_pool.append(QUERY_DEPTH_PARAM + " must contain a an integer (1 or 2). The default value is 1.")
    return 1



def _call_shaper(target_classes, graph, input_fotmat, instantiation_prop,
                 infer_untyped_num, discard_useles_constraints, all_compliant,
                 keep_less_specific, threshold, all_classes_mode, namespaces_dict,
                 namespaces_to_ignore, shape_map, remote_graph, endpoint_sparql,
                 query_depth, disable_comments, namespaces_for_qualifier_props,
                 shape_qualifiers_mode):
    shaper = Shaper(target_classes=target_classes,
                    input_format=input_fotmat,
                    instantiation_property=instantiation_prop,
                    infer_numeric_types_for_untyped_literals=infer_untyped_num,
                    discard_useless_constraints_with_positive_closure=discard_useles_constraints,
                    all_instances_are_compliant_mode=all_compliant,
                    keep_less_specific=keep_less_specific,
                    raw_graph=graph,
                    all_classes_mode=all_classes_mode,
                    namespaces_dict=namespaces_dict,
                    namespaces_to_ignore=namespaces_to_ignore,
                    shape_map_raw=shape_map,
                    url_graph_input=remote_graph,
                    url_endpoint=endpoint_sparql,
                    depth_for_building_subgraph=query_depth,
                    disable_comments=disable_comments,
                    namespaces_for_qualifier_props=namespaces_for_qualifier_props,
                    shape_qualifiers_mode=shape_qualifiers_mode,
                    wikidata_annotation=True
                    )
    result = shaper.shex_graph(acceptance_threshold=threshold, string_output=True)
    return _jsonize_response(result)


def _check_combination_error_input_data(data, error_pool):
    target_params = [TARGET_GRAPH_PARAM, REMOTE_GRAPH_PARAM, ENDPOINT_GRAPH_PARAM]
    counter = 0
    for elem in target_params:
        if elem in data:

            counter += 1
    if counter != 1:
        error_pool.append("You must provide exactly one of the following params: " + ", ".join(target_params) + ".")
        return True
    return False


def _check_combination_error_target_shapes(data, error_pool, all_classes_mode):
    target_params = [TARGET_CLASSES_PARAM, SHAPE_MAP_PARAM]
    counter = 0
    for elem in target_params:
        if elem in data:
            counter += 1
    if counter == 1:
        return False
    if counter == 0 and ALL_CLASSES_MODE_PARAM in data and all_classes_mode == True:
        return False
    error_pool.append("Yoy must provide exactly one of " + ", ".join(target_params) + " or set " + ALL_CLASSES_MODE_PARAM + " to True")
    return True

def _check_all_classes_mode_uncompatibility(data, error_pool, all_classes_mode):
    if all_classes_mode and ENDPOINT_GRAPH_PARAM in data and SHAPE_MAP_PARAM not in data:
        error_pool.append("If you use all classes mode with input via endpoint, you must provide a shape map as well "
                          "to let sheXer knows what part of the graph it should consider.")



################ WS

app = Flask(__name__)

@app.route('/shexer', methods=['POST'])
def shexer():
    error_pool = []
    try:
        data = request.json
        input_fotmat = _parse_input_format(data, error_pool)
        instantiation_prop = _parse_instantiation_prop(data, error_pool)
        infer_untyped_num = _parse_infer_untyped_num(data, error_pool)
        discard_useles_constraints = _parse_discard_useless(data, error_pool)
        all_compliant = _parse_all_compliant(data, error_pool)
        keep_less_specific = _parse_keep_less_specific(data, error_pool)
        threshold = _parse_threshold(data, error_pool)
        all_classes_mode = _parse_all_classes_mode(data, error_pool)
        namespaces_to_ignore = _parse_namespaces_to_ignore(data, error_pool)
        namespaces = _parse_namespaces(data, error_pool)
        query_depth = _parse_query_depth(data, error_pool)
        disable_comments = _parse_disable_comments(data, error_pool)

        shape_qualifiers_mode = _parse_shape_qualifiers_mode(data, error_pool)
        namespaces_for_qualifier_props = _parse_namespaces_for_qualifiers(data, error_pool)

        target_classes = None
        graph = None
        shape_map = None
        remote_graph = None
        endpoint_sparql = None

        err_input = _check_combination_error_input_data(data, error_pool)
        if not err_input:
            graph = _parse_graph(data, error_pool)
            remote_graph = _parse_remote_graph(data, error_pool)
            endpoint_sparql = _parse_endpoint_sparql(data, error_pool)

        err_target = _check_combination_error_target_shapes(data, error_pool, all_classes_mode)
        if not err_target:
            target_classes = _parse_target_classes(data, error_pool)
            shape_map = _parse_shape_map(data, error_pool)
            _check_all_classes_mode_uncompatibility(data, error_pool, all_classes_mode)

        # remote_graph = _parse_remote_graph(data, error_pool)
        # endpoint_sparql = _parse_endpoint_sparql(data, error_pool)

        if len(error_pool) == 0:
            #todo: Call shaper with the new params!
            return _call_shaper(target_classes=target_classes,
                                graph=graph,
                                input_fotmat=input_fotmat,
                                instantiation_prop=instantiation_prop,
                                infer_untyped_num=infer_untyped_num,
                                discard_useles_constraints=discard_useles_constraints,
                                all_compliant=all_compliant,
                                keep_less_specific=keep_less_specific,
                                threshold=threshold,
                                all_classes_mode=all_classes_mode,
                                namespaces_dict=_build_namespaces_dict(namespaces, default_namespaces),
                                endpoint_sparql=endpoint_sparql,
                                shape_map=shape_map,
                                remote_graph=remote_graph,
                                namespaces_to_ignore=namespaces_to_ignore,
                                query_depth=query_depth,
                                disable_comments=disable_comments,
                                namespaces_for_qualifier_props=namespaces_for_qualifier_props,
                                shape_qualifiers_mode=shape_qualifiers_mode
                                )


        else:
           return _return_json_error_pool(error_pool)

    except BaseException as e:
        traceback.print_exc()
        error_pool.append("Internal unexpected server error: " + str(e))
        return _return_json_error_pool(error_pool)

def run():
    port = 80 if len(sys.argv) < 2 else int(sys.argv[1])
    CORS(app)
    app.run(port=port, host=HOST, ssl_context='adhoc')

if __name__ == "__main__":
    run()



import unittest
from shexer.shaper import Shaper
from shexer.utils.uri import prefixize_uri_if_possible, add_corners_if_it_is_an_uri
from shexer.utils.dict import reverse_keys_and_values
from test.const import G1, BASE_FILES, default_namespaces
from test.t_utils import file_vs_str_tunned_comparison, file_vs_str_strip_comparison
from shexer.consts import ALL_EXAMPLES, MIXED_INSTANCES, ABSOLUTE_INSTANCES, RATIO_INSTANCES, CONSTRAINT_EXAMPLES, \
    SHAPE_EXAMPLES, EXAMPLE_CONFORMANCE_PROP, ABSOLUTE_COUNT_PROP, EXTRA_INFO_PROP, FREQ_PROP
import os.path as pth

from shexer.consts import TURTLE, TURTLE_ITER



_BASE_DIR = BASE_FILES + "annotations" + pth.sep  # We just need something with another instantiation property


class TestAnnotations(unittest.TestCase):

    def _namespaces_for_test(self, reverse=False):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        if reverse:
            return reverse_keys_and_values(namespaces)
        return namespaces
    def _example_prop(self):
        return add_corners_if_it_is_an_uri(prefixize_uri_if_possible(
            target_uri=EXAMPLE_CONFORMANCE_PROP,
            namespaces_prefix_dict=self._namespaces_for_test(reverse=False),
            corners=False))
    def _ratio_prop(self):
        return add_corners_if_it_is_an_uri(prefixize_uri_if_possible(
            target_uri=FREQ_PROP,
            namespaces_prefix_dict=self._namespaces_for_test(reverse=False),
            corners=False)
        )

    def _abs_prop(self):
        return add_corners_if_it_is_an_uri(prefixize_uri_if_possible(
            target_uri=ABSOLUTE_COUNT_PROP,
            namespaces_prefix_dict=self._namespaces_for_test(reverse=False),
            corners=False))

    def _extra_prop(self):
        return add_corners_if_it_is_an_uri(prefixize_uri_if_possible(
            target_uri=EXTRA_INFO_PROP,
            namespaces_prefix_dict=self._namespaces_for_test(reverse=False),
            corners=False))

    def test_all_classes_g1_no_annotations(self):
        shaper = Shaper(
            graph_file_input=G1,
            namespaces_dict=self._namespaces_for_test(),
            all_classes_mode=True,
            input_format=TURTLE,
            disable_comments=False,
            generate_annotations=False
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "g1_comments_no_annot.shex",
                                                      str_target=str_result))

    def test_shape_examples(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=SHAPE_EXAMPLES,
            input_format=TURTLE,
            # instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "one_class_shape_examples.shex",
                                                      str_target=str_result))
        self.assertIn(f"}} // {self._example_prop()}", str_result)

    def test_constraint_examples(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=CONSTRAINT_EXAMPLES,
            input_format=TURTLE_ITER,
            # instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_constraint_examples.shex",
                                                     target_str=str_result))

    def test_all_examples(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            # instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_all_examples.shex",
                                                     target_str=str_result))

    def test_ratios(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            # examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=RATIO_INSTANCES,
            disable_comments=False,
            generate_annotations=True
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_ratios.shex",
                                                     target_str=str_result))

    def test_absolutes(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            # examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=ABSOLUTE_INSTANCES,
            disable_comments=False,
            generate_annotations=True
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_absolutes.shex",
                                                     target_str=str_result))

    def test_mixed_report(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            # examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_mixed_report.shex",
                                                     target_str=str_result))

    def test_all_examples_and_stats(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_all_examples_stats.shex",
                                                     target_str=str_result))

    def test_set_extra(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True,
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            # frequency_property="http://weso.esss/RAT",
            extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_all_set_extra.shex",
                                                     target_str=str_result))

    def test_set_ratios(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True,
            # absolute_counts_property="http://weso.esss/ABS",
            # example_conformance_property="http://weso.esss/CONF",
            frequency_property="http://weso.esss/RAT"
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_all_set_ratios.shex",
                                                     target_str=str_result))

    def test_set_examples(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True,
            # absolute_counts_property="http://weso.esss/ABS",
            example_conformance_property="http://weso.esss/CONF"
            # frequency_property="http://weso.esss/RAT"
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_all_set_examples.shex",
                                                     target_str=str_result))

    def test_set_absolutes(self):
        namespaces = default_namespaces()
        namespaces["http://weso.es/shexer/ontology/"] = "shexer"
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "single_class_deterministic_order.ttl",
            namespaces_dict=namespaces,
            all_classes_mode=True,
            examples_mode=ALL_EXAMPLES,
            input_format=TURTLE_ITER,
            instances_report_mode=MIXED_INSTANCES,
            disable_comments=False,
            generate_annotations=True,
            absolute_counts_property="http://weso.esss/ABS"
            # example_conformance_property="http://weso.esss/CONF"
            # frequency_property="http://weso.esss/RAT"
            # extra_info_property="http://weso.esss/EXTRA"
        )
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_strip_comparison(file_path=_BASE_DIR + "one_class_all_set_absolutes.shex",
                                                     target_str=str_result))